import errno
import os
import sys


CAN_USE_PYREPL: bool
FAIL_REASON: str
try:
    if sys.platform == "win32" and sys.getwindowsversion().build < 10586:
        raise RuntimeError("Windows 10 TH2 or later required")
    if not os.isatty(sys.stdin.fileno()):
        raise OSError(errno.ENOTTY, "tty required", "stdin")
    from .simple_interact import check
    if err := check():
        raise RuntimeError(err)
except Exception as e:
    CAN_USE_PYREPL = False
    FAIL_REASON = f"warning: can't use pyrepl: {e}"
else:
    CAN_USE_PYREPL = True
    FAIL_REASON = ""



def interactive_console(mainmodule=None, quiet=False, pythonstartup=False):
    if not CAN_USE_PYREPL:
        if not os.getenv('PYTHON_BASIC_REPL') and FAIL_REASON:
            from .trace import trace
            trace(FAIL_REASON)
            print(FAIL_REASON, file=sys.stderr)
        return sys._baserepl()

    if mainmodule:
        namespace = mainmodule.__dict__
    else:
        import __main__
        namespace = __main__.__dict__
        namespace.pop("__pyrepl_interactive_console", None)

    # sys._baserepl() above does this internally, we do it here
    startup_path = os.getenv("PYTHONSTARTUP")
    if pythonstartup and startup_path:
        sys.audit("cpython.run_startup", startup_path)

        import tokenize
        with tokenize.open(startup_path) as f:
            startup_code = compile(f.read(), startup_path, "exec")
            exec(startup_code, namespace)

    # set sys.{ps1,ps2} just before invoking the interactive interpreter. This
    # mimics what CPython does in pythonrun.c
    if not hasattr(sys, "ps1"):
        sys.ps1 = ">>> "
    if not hasattr(sys, "ps2"):
        sys.ps2 = "... "

    from .console import InteractiveColoredConsole
    from .simple_interact import run_multiline_interactive_console
    console = InteractiveColoredConsole(namespace, filename="<stdin>")
    run_multiline_interactive_console(console)



from _pypy_openssl import lib, ffi


def enum_certificates(store_name):
    """Retrieve certificates from Windows' cert store.

store_name may be one of 'CA', 'ROOT' or 'MY'.  The system may provide
more cert storages, too.  The function returns a list of (bytes,
encoding_type, trust) tuples.  The encoding_type flag can be interpreted
with X509_ASN_ENCODING or PKCS_7_ASN_ENCODING. The trust setting is either
a set of OIDs or the boolean True.
    """
    hStore = lib.CertOpenStore(lib.CERT_STORE_PROV_SYSTEM_A, 0, ffi.NULL,
                               lib.CERT_STORE_READONLY_FLAG | lib.CERT_SYSTEM_STORE_LOCAL_MACHINE,
                               bytes(store_name, "ascii"))
    if hStore == ffi.NULL:
        raise WindowsError(*ffi.getwinerror())
    
    result = []
    pCertCtx = ffi.NULL
    try:
        while True:
            pCertCtx = lib.CertEnumCertificatesInStore(hStore, pCertCtx)
            if pCertCtx == ffi.NULL:
                break
            cert = ffi.buffer(pCertCtx.pbCertEncoded, pCertCtx.cbCertEncoded)[:]
            enc = certEncodingType(pCertCtx.dwCertEncodingType)
            keyusage = parseKeyUsage(pCertCtx, lib.CERT_FIND_PROP_ONLY_ENHKEY_USAGE_FLAG)
            if keyusage is True:
                keyusage = parseKeyUsage(pCertCtx, lib.CERT_FIND_EXT_ONLY_ENHKEY_USAGE_FLAG)
            result.append((cert, enc, keyusage))
    finally:
        if pCertCtx != ffi.NULL:
            lib.CertFreeCertificateContext(pCertCtx)
        if not lib.CertCloseStore(hStore, 0):
            # This error case might shadow another exception.
            raise WindowsError(*ffi.getwinerror())
    return result


def enum_crls(store_name):
    """Retrieve CRLs from Windows' cert store.

store_name may be one of 'CA', 'ROOT' or 'MY'.  The system may provide
more cert storages, too.  The function returns a list of (bytes,
encoding_type) tuples.  The encoding_type flag can be interpreted with
X509_ASN_ENCODING or PKCS_7_ASN_ENCODING."""
    hStore = lib.CertOpenStore(lib.CERT_STORE_PROV_SYSTEM_A, 0, ffi.NULL,
                               lib.CERT_STORE_READONLY_FLAG | lib.CERT_SYSTEM_STORE_LOCAL_MACHINE,
                               bytes(store_name, "ascii"))
    if hStore == ffi.NULL:
        raise WindowsError(*ffi.getwinerror())

    result = []
    pCrlCtx = ffi.NULL
    try:
        while True:
            pCrlCtx = lib.CertEnumCRLsInStore(hStore, pCrlCtx)
            if pCrlCtx == ffi.NULL:
                break
            crl = ffi.buffer(pCrlCtx.pbCrlEncoded, pCrlCtx.cbCrlEncoded)[:]
            enc = certEncodingType(pCrlCtx.dwCertEncodingType)
            result.append((crl, enc))
    finally:
        if pCrlCtx != ffi.NULL:
            lib.CertFreeCRLContext(pCrlCtx)
        if not lib.CertCloseStore(hStore, 0):
            # This error case might shadow another exception.
            raise WindowsError(*ffi.getwinerror())
    return result


def certEncodingType(encodingType):
    if encodingType == lib.X509_ASN_ENCODING:
        return "x509_asn"
    if encodingType == lib.PKCS_7_ASN_ENCODING:
        return "pkcs_7_asn"
    return encodingType

def parseKeyUsage(pCertCtx, flags):
    pSize = ffi.new("DWORD *")
    if not lib.CertGetEnhancedKeyUsage(pCertCtx, flags, ffi.NULL, pSize):
        error_with_message = ffi.getwinerror()
        if error_with_message[0] == lib.CRYPT_E_NOT_FOUND:
            return True
        raise WindowsError(*error_with_message)

    pUsageMem = ffi.new("char[]", pSize[0])
    pUsage = ffi.cast("PCERT_ENHKEY_USAGE", pUsageMem)
    if not lib.CertGetEnhancedKeyUsage(pCertCtx, flags, pUsage, pSize):
        error_with_message = ffi.getwinerror()
        if error_with_message[0] == lib.CRYPT_E_NOT_FOUND:
            return True
        raise WindowsError(*error_with_message)

    retval = set()
    for i in range(pUsage.cUsageIdentifier):
        if pUsage.rgpszUsageIdentifier[i]:
            oid = ffi.string(pUsage.rgpszUsageIdentifier[i]).decode('ascii')
            retval.add(oid)
    return retval



import sys
from distutils import sysconfig

# NOTE: these must be kept on sync with the equivalent defines in hpy.h
HPY_ABI_VERSION = 0
HPY_ABI_VERSION_MINOR = 0
HPY_ABI_TAG = 'hpy%d' % HPY_ABI_VERSION

def parse_ext_suffix(ext_suffix=None):
    """
    Parse EXT_SUFFIX and return abi_tag, ext.

    - abi_tag is a string representing the CPython ABI tag: e.g. 'cp38', 'cp310d', etc.
    - ext is the filename extension, e.g 'so', 'pyd' or 'dylib'
    """
    if ext_suffix is None:
        ext_suffix = sysconfig.get_config_var('EXT_SUFFIX')

    # ext_suffix is something like this. We want to keep the parts which are
    # related to the ABI and remove the parts which are related to the
    # platform:
    #   - linux:   '.cpython-310-x86_64-linux-gnu.so'
    #   - mac:     '.cpython-310-darwin.so'
    #   - win:     '.cp310-win_amd64.pyd'
    #   - pypy:    '.pypy38-pp73-x86_64-linux-gnu.so'
    #   - graalpy: '.graalpy-38-native-x86_64-darwin.dylib'
    assert ext_suffix[0] == '.'
    _, soabi, ext = ext_suffix.split('.')

    # this is very ad-hoc but couldn't find a better way to do it
    parts = soabi.split('-')
    if parts[0].startswith('cpython'):      # cpython on linux, mac
        n = 2
    elif parts[0].startswith('cp'):         # cpython on windows
        n = 1
    elif parts[0].startswith('pypy'):       # pypy
        n = 2
    elif parts[0].startswith('graalpy'):    # graalpy
        n = 3
    else:                                   # none of the above, keep all parts
        n = len(parts)

    abi_tag = '-'.join(parts[:n])
    # on cpython linux/mac, abi is now cpython-310: shorten it ot cp310, like
    # on windows and on wheel tags
    abi_tag = abi_tag.replace('cpython-', 'cp')
    return abi_tag, ext


def get_hpy_ext_suffix(hpy_abi, ext_suffix=None):
    """
    Return the proper filename extension for the given hpy_abi.

    For example with CPython 3.10 on Linux, it will return:

      - universal ==> .hpy0.so
      - hybrid    ==> .hpy0-cp310.so
    """
    assert hpy_abi in ('cpython', 'universal', 'hybrid')
    cpy_abi_tag, ext = parse_ext_suffix(ext_suffix)
    if hpy_abi == 'cpython':
        return sysconfig.get_config_var('EXT_SUFFIX')
    elif hpy_abi == 'universal':
        return '.%s.%s' % (HPY_ABI_TAG, ext)
    else:
        return '.%s-%s.%s' % (HPY_ABI_TAG, cpy_abi_tag, ext)



# -----------------------------------------------------------------
# _ast_gen.py
#
# Generates the AST Node classes from a specification given in
# a configuration file. This module can also be run as a script to
# regenerate c_ast.py from _c_ast.cfg (from the repo root or the
# pycparser/ directory). Use 'make check' to reformat the generated
# file after running this script.
#
# The design of this module was inspired by astgen.py from the
# Python 2.5 code-base.
#
# Eli Bendersky [https://eli.thegreenplace.net/]
# License: BSD
# -----------------------------------------------------------------
from string import Template
import os
from typing import IO


class ASTCodeGenerator:
    def __init__(self, cfg_filename="_c_ast.cfg"):
        """Initialize the code generator from a configuration
        file.
        """
        self.cfg_filename = cfg_filename
        self.node_cfg = [
            NodeCfg(name, contents)
            for (name, contents) in self.parse_cfgfile(cfg_filename)
        ]

    def generate(self, file: IO[str]) -> None:
        """Generates the code into file, an open file buffer."""
        src = Template(_PROLOGUE_COMMENT).substitute(cfg_filename=self.cfg_filename)

        src += _PROLOGUE_CODE
        for node_cfg in self.node_cfg:
            src += node_cfg.generate_source() + "\n\n"

        file.write(src)

    def parse_cfgfile(self, filename):
        """Parse the configuration file and yield pairs of
        (name, contents) for each node.
        """
        with open(filename, "r") as f:
            for line in f:
                line = line.strip()
                if not line or line.startswith("#"):
                    continue
                colon_i = line.find(":")
                lbracket_i = line.find("[")
                rbracket_i = line.find("]")
                if colon_i < 1 or lbracket_i <= colon_i or rbracket_i <= lbracket_i:
                    raise RuntimeError(f"Invalid line in {filename}:\n{line}\n")

                name = line[:colon_i]
                val = line[lbracket_i + 1 : rbracket_i]
                vallist = [v.strip() for v in val.split(",")] if val else []
                yield name, vallist


class NodeCfg:
    """Node configuration.

    name: node name
    contents: a list of contents - attributes and child nodes
    See comment at the top of the configuration file for details.
    """

    def __init__(self, name, contents):
        self.name = name
        self.all_entries = []
        self.attr = []
        self.child = []
        self.seq_child = []

        for entry in contents:
            clean_entry = entry.rstrip("*")
            self.all_entries.append(clean_entry)

            if entry.endswith("**"):
                self.seq_child.append(clean_entry)
            elif entry.endswith("*"):
                self.child.append(clean_entry)
            else:
                self.attr.append(entry)

    def generate_source(self):
        src = self._gen_init()
        src += "\n" + self._gen_children()
        src += "\n" + self._gen_iter()
        src += "\n" + self._gen_attr_names()
        return src

    def _gen_init(self):
        src = f"class {self.name}(Node):\n"

        if self.all_entries:
            args = ", ".join(self.all_entries)
            slots = ", ".join(f"'{e}'" for e in self.all_entries)
            slots += ", 'coord', '__weakref__'"
            arglist = f"(self, {args}, coord=None)"
        else:
            slots = "'coord', '__weakref__'"
            arglist = "(self, coord=None)"

        src += f"    __slots__ = ({slots})\n"
        src += f"    def __init__{arglist}:\n"

        for name in self.all_entries + ["coord"]:
            src += f"        self.{name} = {name}\n"

        return src

    def _gen_children(self):
        src = "    def children(self):\n"

        if self.all_entries:
            src += "        nodelist = []\n"

            for child in self.child:
                src += f"        if self.{child} is not None:\n"
                src += f'            nodelist.append(("{child}", self.{child}))\n'

            for seq_child in self.seq_child:
                src += f"        for i, child in enumerate(self.{seq_child} or []):\n"
                src += f'            nodelist.append((f"{seq_child}[{{i}}]", child))\n'

            src += "        return tuple(nodelist)\n"
        else:
            src += "        return ()\n"

        return src

    def _gen_iter(self):
        src = "    def __iter__(self):\n"

        if self.all_entries:
            for child in self.child:
                src += f"        if self.{child} is not None:\n"
                src += f"            yield self.{child}\n"

            for seq_child in self.seq_child:
                src += f"        for child in (self.{seq_child} or []):\n"
                src += "            yield child\n"

            if not (self.child or self.seq_child):
                # Empty generator
                src += "        return\n" + "        yield\n"
        else:
            # Empty generator
            src += "        return\n" + "        yield\n"

        return src

    def _gen_attr_names(self):
        src = "    attr_names = (" + "".join(f"{nm!r}, " for nm in self.attr) + ")"
        return src


_PROLOGUE_COMMENT = r"""#-----------------------------------------------------------------
# ** ATTENTION **
# This code was automatically generated from _c_ast.cfg
#
# Do not modify it directly. Modify the configuration file and
# run the generator again.
# ** ** *** ** **
#
# pycparser: c_ast.py
#
# AST Node classes.
#
# Eli Bendersky [https://eli.thegreenplace.net/]
# License: BSD
#-----------------------------------------------------------------

"""
_PROLOGUE_CODE = r'''
import sys
from typing import Any, ClassVar, IO, Optional

def _repr(obj):
    """
    Get the representation of an object, with dedicated pprint-like format for lists.
    """
    if isinstance(obj, list):
        return '[' + (',\n '.join((_repr(e).replace('\n', '\n ') for e in obj))) + '\n]'
    else:
        return repr(obj)

class Node:
    __slots__ = ()
    """ Abstract base class for AST nodes.
    """
    attr_names: ClassVar[tuple[str, ...]] = ()
    coord: Optional[Any]
    def __repr__(self):
        """ Generates a python representation of the current node
        """
        result = self.__class__.__name__ + '('

        indent = ''
        separator = ''
        for name in self.__slots__[:-2]:
            result += separator
            result += indent
            result += name + '=' + (_repr(getattr(self, name)).replace('\n', '\n  ' + (' ' * (len(name) + len(self.__class__.__name__)))))

            separator = ','
            indent = '\n ' + (' ' * len(self.__class__.__name__))

        result += indent + ')'

        return result

    def children(self):
        """ A sequence of all children that are Nodes
        """
        pass

    def show(
        self,
        buf: IO[str] = sys.stdout,
        offset: int = 0,
        attrnames: bool = False,
        showemptyattrs: bool = True,
        nodenames: bool = False,
        showcoord: bool = False,
        _my_node_name: Optional[str] = None,
    ):
        """ Pretty print the Node and all its attributes and
            children (recursively) to a buffer.

            buf:
                Open IO buffer into which the Node is printed.

            offset:
                Initial offset (amount of leading spaces)

            attrnames:
                True if you want to see the attribute names in
                name=value pairs. False to only see the values.

            showemptyattrs:
                False if you want to suppress printing empty attributes.

            nodenames:
                True if you want to see the actual node names
                within their parents.

            showcoord:
                Do you want the coordinates of each Node to be
                displayed.
        """
        lead = ' ' * offset
        if nodenames and _my_node_name is not None:
            buf.write(lead + self.__class__.__name__+ ' <' + _my_node_name + '>: ')
        else:
            buf.write(lead + self.__class__.__name__+ ': ')

        if self.attr_names:
            def is_empty(v):
                v is None or (hasattr(v, '__len__') and len(v) == 0)
            nvlist = [(n, getattr(self,n)) for n in self.attr_names \
                        if showemptyattrs or not is_empty(getattr(self,n))]
            if attrnames:
                attrstr = ', '.join(f'{name}={value}' for name, value in nvlist)
            else:
                attrstr = ', '.join(f'{value}' for _, value in nvlist)
            buf.write(attrstr)

        if showcoord:
            buf.write(f' (at {self.coord})')
        buf.write('\n')

        for (child_name, child) in self.children():
            child.show(
                buf,
                offset=offset + 2,
                attrnames=attrnames,
                showemptyattrs=showemptyattrs,
                nodenames=nodenames,
                showcoord=showcoord,
                _my_node_name=child_name)


class NodeVisitor:
    """ A base NodeVisitor class for visiting c_ast nodes.
        Subclass it and define your own visit_XXX methods, where
        XXX is the class name you want to visit with these
        methods.

        For example:

        class ConstantVisitor(NodeVisitor):
            def __init__(self):
                self.values = []

            def visit_Constant(self, node):
                self.values.append(node.value)

        Creates a list of values of all the constant nodes
        encountered below the given node. To use it:

        cv = ConstantVisitor()
        cv.visit(node)

        Notes:

        *   generic_visit() will be called for AST nodes for which
            no visit_XXX method was defined.
        *   The children of nodes for which a visit_XXX was
            defined will not be visited - if you need this, call
            generic_visit() on the node.
            You can use:
                NodeVisitor.generic_visit(self, node)
        *   Modeled after Python's own AST visiting facilities
            (the ast module of Python 3.0)
    """

    _method_cache = None

    def visit(self, node: Node):
        """ Visit a node.
        """

        if self._method_cache is None:
            self._method_cache = {}

        visitor = self._method_cache.get(node.__class__.__name__, None)
        if visitor is None:
            method = 'visit_' + node.__class__.__name__
            visitor = getattr(self, method, self.generic_visit)
            self._method_cache[node.__class__.__name__] = visitor

        return visitor(node)

    def generic_visit(self, node: Node):
        """ Called if no explicit visitor function exists for a
            node. Implements preorder visiting of the node.
        """
        for _, c in node.children():
            self.visit(c)

'''


if __name__ == "__main__":
    base_dir = os.path.dirname(os.path.abspath(__file__))
    cfg_path = os.path.join(base_dir, "_c_ast.cfg")
    out_path = os.path.join(base_dir, "c_ast.py")
    ast_gen = ASTCodeGenerator(cfg_path)
    with open(out_path, "w") as out:
        ast_gen.generate(out)



from cffi import FFI

ffi = FFI()

ffi.set_source("_pwdgrp_cffi", """
#include <sys/types.h>
#include <pwd.h>
#include <grp.h>
""")


ffi.cdef("""

typedef int... uid_t;
typedef int... gid_t;

struct passwd {
    char *pw_name;
    char *pw_passwd;
    uid_t pw_uid;
    gid_t pw_gid;
    char *pw_gecos;
    char *pw_dir;
    char *pw_shell;
    ...;
};

struct group {
    char *gr_name;       /* group name */
    char *gr_passwd;     /* group password */
    gid_t gr_gid;        /* group ID */
    char **gr_mem;        /* group members */
};

struct passwd *getpwuid(uid_t uid);
struct passwd *getpwnam(const char *name);

struct passwd *getpwent(void);
void setpwent(void);
void endpwent(void);

struct group *getgrgid(gid_t gid);
struct group *getgrnam(const char *name);

struct group *getgrent(void);
void setgrent(void);
void endgrent(void);

""")


if __name__ == "__main__":
    ffi.compile()



import sys
import unittest
from test import support
from test.support import import_helper
from test.support import check_sanitizer

if check_sanitizer(address=True, memory=True):
    raise unittest.SkipTest("Tests involvin libX11 can SEGFAULT on ASAN/MSAN builds")


# Skip this test if the _tkinter module wasn't built.
_tkinter = import_helper.import_module('_tkinter')

# Skip test if tk cannot be initialized.
support.requires('gui')

# Suppress the deprecation warning
tix = import_helper.import_module('tkinter.tix', deprecated=True)
from tkinter import TclError


class TestTix(unittest.TestCase):

    def setUp(self):
        try:
            self.root = tix.Tk()
        except TclError:
            if sys.platform.startswith('win'):
                self.fail('Tix should always be available on Windows')
            self.skipTest('Tix not available')
        else:
            self.addCleanup(self.root.destroy)

    def test_tix_available(self):
        # this test is just here to make setUp run
        pass


if __name__ == '__main__':
    unittest.main()



"""
A testcase which accesses *values* in a dll.
"""

import _imp
import importlib.util
import unittest
import sys
from ctypes import *
from test.support import import_helper, cpython_only

import _ctypes_test

class ValuesTestCase(unittest.TestCase):

    def test_an_integer(self):
        # This test checks and changes an integer stored inside the
        # _ctypes_test dll/shared lib.
        ctdll = CDLL(_ctypes_test.__file__)
        an_integer = c_int.in_dll(ctdll, "an_integer")
        x = an_integer.value
        self.assertEqual(x, ctdll.get_an_integer())
        an_integer.value *= 2
        self.assertEqual(x*2, ctdll.get_an_integer())
        # To avoid test failures when this test is repeated several
        # times the original value must be restored
        an_integer.value = x
        self.assertEqual(x, ctdll.get_an_integer())

    def test_undefined(self):
        ctdll = CDLL(_ctypes_test.__file__)
        self.assertRaises(ValueError, c_int.in_dll, ctdll, "Undefined_Symbol")

@cpython_only
class PythonValuesTestCase(unittest.TestCase):
    """This test only works when python itself is a dll/shared library"""

    def test_optimizeflag(self):
        # This test accesses the Py_OptimizeFlag integer, which is
        # exported by the Python dll and should match the sys.flags value

        opt = c_int.in_dll(pythonapi, "Py_OptimizeFlag").value
        self.assertEqual(opt, sys.flags.optimize)

    def test_frozentable(self):
        # Python exports a PyImport_FrozenModules symbol. This is a
        # pointer to an array of struct _frozen entries.  The end of the
        # array is marked by an entry containing a NULL name and zero
        # size.

        # In standard Python, this table contains a __hello__
        # module, and a __phello__ package containing a spam
        # module.
        class struct_frozen(Structure):
            _fields_ = [("name", c_char_p),
                        ("code", POINTER(c_ubyte)),
                        ("size", c_int),
                        ("is_package", c_int),
                        ("get_code", POINTER(c_ubyte)),  # Function ptr
                        ]
        FrozenTable = POINTER(struct_frozen)

        modules = []
        for group in ["Bootstrap", "Stdlib", "Test"]:
            ft = FrozenTable.in_dll(pythonapi, f"_PyImport_Frozen{group}")
            # ft is a pointer to the struct_frozen entries:
            for entry in ft:
                # This is dangerous. We *can* iterate over a pointer, but
                # the loop will not terminate (maybe with an access
                # violation;-) because the pointer instance has no size.
                if entry.name is None:
                    break
                modname = entry.name.decode("ascii")
                modules.append(modname)
                with self.subTest(modname):
                    if entry.size != 0:
                        # Do a sanity check on entry.size and entry.code.
                        self.assertGreater(abs(entry.size), 10)
                        self.assertTrue([entry.code[i] for i in range(abs(entry.size))])
                    # Check the module's package-ness.
                    with import_helper.frozen_modules():
                        spec = importlib.util.find_spec(modname)
                    if entry.is_package:
                        # It's a package.
                        self.assertIsNotNone(spec.submodule_search_locations)
                    else:
                        self.assertIsNone(spec.submodule_search_locations)

        with import_helper.frozen_modules():
            expected = _imp._frozen_module_names()
        self.maxDiff = None
        self.assertEqual(modules, expected,
                         "_PyImport_FrozenBootstrap example "
                         "in Doc/library/ctypes.rst may be out of date")

        from ctypes import _pointer_type_cache
        del _pointer_type_cache[struct_frozen]

    def test_undefined(self):
        self.assertRaises(ValueError, c_int.in_dll, pythonapi,
                          "Undefined_Symbol")

if __name__ == '__main__':
    unittest.main()



# -*- encoding: utf-8 -*-

# This is a package that contains a number of modules that are used to
# test import from the source files that have different encodings.
# This file (the __init__ module of the package), is encoded in utf-8
# and contains a list of strings from various unicode planes that are
# encoded differently to compare them to the same strings encoded
# differently in submodules.  The following list, test_strings,
# contains a list of tuples. The first element of each tuple is the
# suffix that should be prepended with 'module_' to arrive at the
# encoded submodule name, the second item is the encoding and the last
# is the test string.  The same string is assigned to the variable
# named 'test' inside the submodule.  If the decoding of modules works
# correctly, from module_xyz import test should result in the same
# string as listed below in the 'xyz' entry.

# module, encoding, test string
test_strings = (
    ('iso_8859_1', 'iso-8859-1', "Les hommes ont oublié cette vérité, "
     "dit le renard. Mais tu ne dois pas l'oublier. Tu deviens "
     "responsable pour toujours de ce que tu as apprivoisé."),
    ('koi8_r', 'koi8-r', "Познание бесконечности требует бесконечного времени.")
)



import os
import unittest
from test.support.import_helper import import_module

termios = import_module('termios')
tty = import_module('tty')


@unittest.skipUnless(hasattr(os, 'openpty'), "need os.openpty()")
class TestTty(unittest.TestCase):

    def setUp(self):
        master_fd, self.fd = os.openpty()
        self.addCleanup(os.close, master_fd)
        self.stream = self.enterContext(open(self.fd, 'wb', buffering=0))
        self.fd = self.stream.fileno()
        self.mode = termios.tcgetattr(self.fd)
        self.addCleanup(termios.tcsetattr, self.fd, termios.TCSANOW, self.mode)
        self.addCleanup(termios.tcsetattr, self.fd, termios.TCSAFLUSH, self.mode)

    def check_cbreak(self, mode):
        self.assertEqual(mode[3] & termios.ECHO, 0)
        self.assertEqual(mode[3] & termios.ICANON, 0)
        self.assertEqual(mode[6][termios.VMIN], 1)
        self.assertEqual(mode[6][termios.VTIME], 0)

    def check_raw(self, mode):
        self.assertEqual(mode[0] & termios.ICRNL, 0)
        self.check_cbreak(mode)
        self.assertEqual(mode[0] & termios.ISTRIP, 0)
        self.assertEqual(mode[0] & termios.ICRNL, 0)
        self.assertEqual(mode[1] & termios.OPOST, 0)
        self.assertEqual(mode[2] & termios.PARENB, termios.CS8 & termios.PARENB)
        self.assertEqual(mode[2] & termios.CSIZE, termios.CS8 & termios.CSIZE)
        self.assertEqual(mode[2] & termios.CS8, termios.CS8)
        self.assertEqual(mode[3] & termios.ECHO, 0)
        self.assertEqual(mode[3] & termios.ICANON, 0)
        self.assertEqual(mode[3] & termios.ISIG, 0)
        self.assertEqual(mode[6][termios.VMIN], 1)
        self.assertEqual(mode[6][termios.VTIME], 0)

    def test_setraw(self):
        tty.setraw(self.fd)
        mode = termios.tcgetattr(self.fd)
        self.check_raw(mode)
        tty.setraw(self.fd, termios.TCSANOW)
        tty.setraw(self.stream)
        tty.setraw(fd=self.fd, when=termios.TCSANOW)

    def test_setcbreak(self):
        tty.setcbreak(self.fd)
        mode = termios.tcgetattr(self.fd)
        self.check_cbreak(mode)
        tty.setcbreak(self.fd, termios.TCSANOW)
        tty.setcbreak(self.stream)
        tty.setcbreak(fd=self.fd, when=termios.TCSANOW)


if __name__ == '__main__':
    unittest.main()
